###############################
##  NIST PT Analysis Program  #
##  Last modified 4/18/2011.  #
###############################

# Input directory where all data and 
# output files are stored.

path = "C:\\Documents and Settings\\NIST PT Folder\\"

# Input data file name - without extention.
# We assume the file has a .csv extention.

id = "LL-120 55481"

####################################

# Create file names for graph files.
pchart = paste(path,id,"_pdiff",sep="")
dchart = paste(path,id,"_dist",sep="")

# Read comma-delimited ASCII data file.
nf = paste(path,id,".csv",sep="")
mall <- matrix(scan(nf,skip=3,sep=","),byrow=T,ncol=9)

# Missing values (-999) may occur for columns 4-8.
# We only use columns 1-3.

# Save new value in separate vector.
# The data for the DUT must be in the first row.
newm = mall[1,]

# Save mall without first observation.
mall = mall[-1,]

# eliminate observations with zero variance.
m = mall[mall[,2]>0,]

xbar = m[,1]
std = m[,2]
n = m[,3]
stderr = std/sqrt(n)

#############################################
#  Run Antonio's laboratory effects model.  #
#############################################

negloglik.R = function (par,x,u,nu)
{
n=length(x)
mu=par[1]
tau = par[2]
sigma = par[3:(n+2)]
if (any(c(sigma,tau) <=0)) {return(Inf)} else{
  return(-1*
  (sum(dnorm(x,mean=mu,sd=sqrt(tau^2 + sigma^2), log=T))
   + sum(log(nu/sigma^2))
   + sum(dchisq(nu*(u/sigma)^2, df=nu, log=T)))) }
}

# algorithm doesn't work when u=0 (variance=0).
mtemp = m
xt = mtemp[,1]
n = length(xt)
u = mtemp[,2]/sqrt(mtemp[,3])
nu = mtemp[,3]-1

o = optim(par=c(median(xt), mad(xt), u),
    fn = negloglik.R, method="Nelder-Mead",
    hessian=TRUE, x=xt, u=u, nu=nu)

#names(o$par) = c("mu", "tau", paste("sigma",sep=""))
z = round(rbind(estimate=o$par, 
      uncertainty = sqrt(diag(solve(o$hessian)))),2)
z

pernew = 100*(newm[1]-z[1,1])/z[1,1]
perd = 100*(xbar - z[1,1])/z[1,1]
plim = 100*z[1,2]/z[1,1]
pmax = max(perd)

###################################################
#  Plot percent deviations from consensus value.  #
###################################################

# Save graph in postscript file.
# fname = paste(pchart,".ps",sep="")
# postscript(file=fname)

# Save graph in jpg file.
fname = paste(pchart,".jpg",sep="")
jpeg(filename=fname, quality=100)

par(mfrow=c(1,1), cex=1.3)

plot(perd,ylim=c(-1.05*pmax,1.05*pmax),type="h",axes=FALSE,
     ylab="Percent Difference from Consensus Value",
     xlab=paste(id, " Results (n = ", length(xbar), ")", sep=""))
axis(1,labels=FALSE,tick=FALSE)
axis(2)
box()
abline(h=0,col="blue")

abline(h=2*plim,col="blue")
abline(h=-2*plim,col="blue")

abline(h=3*plim,col="red")
abline(h=-3*plim,col="red")

#points(0,pernew,pch=19,col="red")
segments(0,0,0,pernew,col="red")
title(main=paste("Consensus Value =", z[1,1], "J,", 
      "Std. Dev. among machines =", z[1,2], "J"),
      cex.main=0.95, font.main=1)

text(x=length(xbar)-1, y=2*plim,
     "2s", pos=1, offset=0.25, cex=0.9, col="blue")
text(x=length(xbar)-1, y=-2*plim, 
     "2s", pos=1, offset=0.25, cex=0.9, col="blue")

text(x=length(xbar)-1, y=3*plim, 
     "3s", pos=3, offset=0.25, cex=0.9, col="red")
text(x=length(xbar)-1, y=-3*plim, 
     "3s", pos=1, offset=0.25, cex=0.9, col="red")

dev.off()

##########################################################
#  Combine raw data with percent differences and         #
#  subset the cases where percent difference le 2*plim.  #
##########################################################

r = cbind(m[,1],m[,2],m[,3],perd)
rsub = r[abs(r[,4])<=2*plim,]

###################################################
#  Plot xbar vs s.d. - with martinal histograms.  #
###################################################

# Web site for plot with marginal histograms.
# http://addictedtor.free.fr/graphiques/graphcode.php?graph=78

# Save graph in postscript file.
# fname = paste(dchart,".ps",sep="")
# postscript(file=fname)

# Save graph in jpg file.
fname = paste(dchart,".jpg",sep="")
jpeg(filename=fname, quality=100)

def.par <- par(no.readonly = TRUE) # save default, for resetting...

x <- rsub[,1]
y <- rsub[,2]
xhist <- hist(x, plot=FALSE,breaks=20)
yhist <- hist(y, plot=FALSE,breaks=10)
top <- max(c(xhist$counts, yhist$counts))
xrange = c(min(xhist$breaks),max(xhist$breaks))
yrange = c(0,max(yhist$breaks))
nf <- layout(matrix(c(2,4,1,3),2,2,byrow=TRUE), c(3,1), c(1,3), TRUE)
#layout.show(nf)

xp025 = quantile(x,0.025)
xp975 = quantile(x,0.975)
nint = length(xhist$counts)
ilen = xhist$mids[2] - xhist$mids[1]
xmin = xhist$mids[1] - ilen/2
xmax = xhist$mids[nint] + ilen/2
np025 = (xp025 - xmin)*nint/(xmax-xmin)
np975 = (xp975 - xmin)*nint/(xmax-xmin)

sp95 = quantile(y,.95)
yint = length(yhist$counts)
ylen = yhist$mids[2] - yhist$mids[1]
ymin = yhist$mids[1] - ylen/2
ymax = yhist$mids[yint] + ylen/2
yp95 = (sp95 - ymin)*yint/(ymax-ymin)

par(mar=c(5,4,1,1), cex=1.3)
plot(x, y, xlim=xrange, ylim=yrange, xlab="Average, J", 
     ylab="Standard Deviation, J")
points(newm[1],newm[2],pch=19,col="red",cex=1.25)
#abline(v=quantile(x,0.025),col="blue")
#abline(v=quantile(x,0.975),col="blue")
#abline(h=quantile(y,0.950),col="blue")
par(mar=c(0,4,1,1))
barplot(xhist$counts, axes=FALSE, ylim=c(0, top), space=0, beside=TRUE)
abline(v=np025,col="blue")
abline(v=np975,col="blue")
par(mar=c(5,0,1,1))
barplot(yhist$counts, axes=FALSE, xlim=c(0, top), space=0, horiz=TRUE)
abline(h=yp95,col="blue")
par(mar=c(0,0,0,0))
plot(c(1:100),c(1:100),type="n", axes=FALSE, xlab="", ylab="")
text(-10,80, id, pos=4, cex=1)
text(-10,65, "Blue lines:", pos=4, cex=0.8)
text(-10,50, "Top: 2.5 %, 97.5 %", pos=4, cex=0.8)
text(-10,35, "Right: 95.0 %", pos=4, cex=0.8)

par(def.par,mar=c(5, 4, 4, 2))
dev.off()

#######################################
#  Generate ASCII file with results.  #
#######################################
zscore = round((newm[1] - z[1,1])/z[1,2],2)

r0 = "  "
r1 = paste("Results for test", id)
r2 = "Verification Test Results (J)"
r3 = paste(newm[4:(3+newm[3])],collapse="  ")
r4 = paste("Mean =", round(newm[1],1), "J")
r5 = paste("Standard deviation =", round(newm[2],1), "J")
r6 = paste("Z-score =", zscore, "J")
r7 = paste("Number of machines =", length(xbar))
r8 = paste("Consensus Value =", z[1,1], "J")
r9 = paste("Std. Dev. Among Machines =", z[1,2], "J")
r10 = paste("One Std. Dev. Limits (J) = (", 
     z[1,1]-z[1,2], ",", z[1,1]+z[1,2], ")", sep="")
r11 = paste("Two Std. Dev. Limits (J) = (", 
     z[1,1]-2*z[1,2], ",", z[1,1]+2*z[1,2], ")", sep="")
r12 = "Quantiles of the distribution of machine averages"
xs = summary(xbar)
r13 = paste("       Minimum =", xs[1], "J")
r14 = paste("0.025 Quantile =", round(xp025,2), "J")
r15 = paste("0.25  Quantile =", xs[2], "J")
r16 = paste("        Median =", xs[3], "J")
r17 = paste("0.75  Quantile =", xs[5], "J")
r18 = paste("0.975 Quantile =", round(xp975,2), "J")
r19 = paste("       Maximum =", xs[6], "J")

r20 = paste("0.95 Quantile of machine standard deviations =",
      round(sp95,2), "J")

rbind(r1,r2,r3,r4,r5,r6,r7,r8,r9,r10,r11,r12,r13,r14,r15,
      r16,r17,r18,r19,r20)

# Save results to a text file.
junk = list(r1,r0,r2,r3,r0,r4,r5,r6,r0,r7,r8,r9,r0,r10,r11,r0,r12,
       r13,r14,r15,r16,r17,r18,r19,r0,r20)
lapply(junk, write, paste(path,id,"_results.txt",sep=""), 
       append=TRUE, ncolumns=1000)

####################################

